// Copyright (C) 2019 Intel Corporation
//
// SPDX-License-Identifier: Apache-2.0

#include "hddl-bsl.h"
#include "stdio.h"

void hddl_get_error_string(int ec) {
  char* error_name = NULL;
#undef MAKE_ERROR_NAME_CASE
#define MAKE_ERROR_NAME_CASE(error_code) \
  case error_code:                       \
    error_name = #error_code;            \
    break

  switch (ec) {
    MAKE_ERROR_NAME_CASE(BSL_SUCCESS);

    MAKE_ERROR_NAME_CASE(BSL_ERROR_RESET_ALL_FAILED);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_RESET_FAILED);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_DROP_FAILED);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_UNSUPPORTED_FUNCTION);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_INVALID_DEVICE_TYPE);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_INVALID_CFG_FILE);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_CFG_OPEN_FAILED);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_HDDL_INSTALL_DIR_NOT_PROVIDED);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_HDDL_INSTALL_DIR_TOO_LONG);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_HDDL_INSTALL_DIR_NOT_DIR);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_MEMORY_ALLOC_FAILED);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_STRNCPY);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_CALLBACK_NOT_FOUND);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_IOCTL_FAIL);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_MEMORY_ERROR);

    MAKE_ERROR_NAME_CASE(BSL_ERROR_INVALID_DEVICE_ID);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_NO_DEVICE_FOUND);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_NO_HID_DEVICE_FOUND);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_NO_IOEXPANDER_DEVICE_FOUND);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_NO_MCU_DEVICE_FOUND);

    MAKE_ERROR_NAME_CASE(BSL_ERROR_HID_API_OPEN_FAILED);

    MAKE_ERROR_NAME_CASE(BSL_ERROR_OPEN_I2C_PERMISSION_DENIED);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_OPEN_I2C_NO_SUCH_FILE);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_OPEN_I2C_OUT_OF_RANGE);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_OPEN_I2C);

    MAKE_ERROR_NAME_CASE(BSL_ERROR_I2C_INVALID_ADDRESS);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_GATHER_ADAPTERS);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_I2C_BUS_NAME_CONFLICT);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_I2C_BUS_NOT_FOUND_CHECK_I801_DRIVER);

    MAKE_ERROR_NAME_CASE(BSL_ERROR_I2C_BUS_CreateFileW);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_I2C_BUS_READ_FAIL);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_I2C_BUS_WRITE_FAIL);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_I2C_CHECK_FUNC_FIRST_ERROR);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_I2C_CHECK_FUNC_SECOND_ERROR);

    MAKE_ERROR_NAME_CASE(BSL_ERROR_PCHGPIO_PCH_NOT_FOUND);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_PCHGPIO_SBREG_BAR_NOT_IMPLEMENT);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_PCHGPIO_CANT_HIDDEN_PCH);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_PCHGPIO_CANT_OPEN_DEVMEM);
    MAKE_ERROR_NAME_CASE(BSL_ERROR_PCHGPIO_MMAP_FAILED);

    MAKE_ERROR_NAME_CASE(BSL_ERROR_CFG_PARSING_GET_NULL_OBJECT);

    default:
      printf("This error code is not defined by bsl_error_code\n");
  }

  if (error_name) {
    printf("%s\n", error_name);
  }
}